using System;
using System.Collections;
using System.Collections.Generic;
using System.Security.Cryptography;
using UnityEngine;
using Random = System.Random;


public class Player
{
    private int _health;
    private bool _alive;
    private const int MaxHealth = 20; // Static, only here to be configurable
    private bool _isDefending;
    private bool _isCharging;

    public Player(int health)
    {
        SetHealth(health);
        _isDefending = false;
        _alive = true;
        _isCharging = false;
    }

    public int GetHealth()
    {
        return _health;
    }

    public void SetHealth(int newHealth)
    {
        if (newHealth > MaxHealth)
        {
            _health = MaxHealth;
            return;
        }
        _health = newHealth;
    }

    public void DealDamage(int damage)
    {
        if (_isDefending)
        {
            _isDefending = false;
            return;
        }

        if (_health - damage <= 0)
        {
            _alive = false;
            _health = 0;
            return;
        }
        _health -= damage;
    }

    public void SetDefending()
    {
        _isDefending = true;
    }

    public void Heal(int boost)
    {
        SetHealth(_health + boost);
    }

    public bool IsAlive()
    {
        return _alive;
    }

    public void SetCharging(bool charging)
    {
        _isCharging = charging;
    }

    public bool IsCharging()
    {
        return _isCharging;
    }
}

public enum GameState
{
    PRINT,
    PLAYER_TURN, 
    AI_TURN, 
    END
}

public class Intro : MonoBehaviour
{
    private Player _ai;
    private Player _player;
    private Random _randomNumberGenerator;
    private GameState _currentGameState; 

    // Start is called before the first frame update
    void Start()
    {
        _randomNumberGenerator = new Random();
        _ai = new Player(20);
        _player = new Player(20);
        _currentGameState = GameState.PRINT;
    }

    // Update is called once per frame
    void Update()
    {
        switch (_currentGameState)
        {
            case GameState.PRINT:
                PrintOptionsForPlayer();
                _currentGameState = GameState.PLAYER_TURN;
                break;
            case GameState.PLAYER_TURN:
                if (_player.IsCharging())
                {
                    HandlePlayerSpecialAttack();
                    break;
                }
                if (Input.GetKeyDown(KeyCode.Alpha1))
                {
                    HandleAttackEvent();
                } 
                else if (Input.GetKeyDown(KeyCode.Alpha2))
                {
                    HandleHealEvent();
                } 
                else if (Input.GetKeyDown(KeyCode.Alpha3))
                {
                    HandleDefendEvent();
                } 
                else if (Input.GetKeyDown(KeyCode.Alpha4))
                {
                    HandlePlayerCharge();
                }
                break;
            case GameState.AI_TURN:

                if (_ai.IsCharging())
                {
                    HandleAISpecialAttack();
                    return;
                }
                
                var decider = _randomNumberGenerator.Next(0, 101);
                if (decider <= 35)
                {
                    HandleAICharge();
                } 
                else if (decider <= 65)
                {
                    HandleAIAttack();
                }
                break;
            case GameState.END: 
                break;
            default:
                throw new ArgumentOutOfRangeException();
        }
    }
    
    private void PrintOptionsForPlayer()
    {
        Debug.Log("Your Turn! Please choose an Action!");
        Debug.Log("Press 1 to Attack, 2 to Heal, 3 to Defend or 4 to charge Special Attack!");
    }

    private void HandleAttackEvent()
    {
        Attack("Player", _ai, "AI", 1, 3);
                    
        if (!_ai.IsAlive())
        {
            _currentGameState = GameState.END; 
            Debug.Log("You won! You killed the AI!");
            return;
        }
        _currentGameState = GameState.AI_TURN;
    }

    private void Attack(string attacker, Player target, string targetName, int lowerBound, int upperBound)
    {
        var damage = _randomNumberGenerator.Next(lowerBound, upperBound);
        target.DealDamage(damage);
        Debug.Log(attacker + " dealt " + damage + " Damage to " + targetName + "! " + target.GetHealth() + "HP left!");
    }

    private void HandleHealEvent()
    {
        _player.Heal(2);
        _currentGameState = GameState.AI_TURN;
    }

    private void HandleDefendEvent()
    {
        _player.SetDefending();
        _currentGameState = GameState.AI_TURN;
    }

    private void HandlePlayerCharge()
    {
        _player.SetCharging(true);
        Debug.Log("You are now Charging special Attack...");
        _currentGameState = GameState.PRINT;
    }

    private void HandlePlayerSpecialAttack()
    {
        _player.SetCharging(false);
        var random = _randomNumberGenerator.Next(101);
        if (21 > random)
        {
            _ai.DealDamage(int.MaxValue);
            if (!_ai.IsAlive())
            {
                _currentGameState = GameState.END; 
                Debug.Log("You Won! You power Punched the AI out of existence!");
                return;
            }
            Debug.Log("Oh No! The AI Defended against your Fatal blow!");
        }
        else
        {
            Debug.Log("Smashing Hammer Attack failed! You Smacked your own Hand instead! -2HP | " + _player.GetHealth() + "HP Left!");
            _ai.DealDamage(2);
            if (!_ai.IsAlive())
            {
                Debug.Log("AI was killed by their own Incompetence! You Won!");
                _currentGameState = GameState.END;
                return;
            }
        }
        _currentGameState = GameState.PRINT;
    }

    private void HandleAIAttack()
    {
        Attack("AI", _player, "Player", 2, 5);
        if (!_player.IsAlive())
        {
            _currentGameState = GameState.END; 
            Debug.Log("You Lost! You were killed by the AI!");
            return;
        }
        _currentGameState = GameState.PRINT;
    }

    private void HandleAISpecialAttack()
    {
        _ai.SetCharging(false);
        var random = _randomNumberGenerator.Next(101);
        if (21 > random)
        {
            _player.DealDamage(int.MaxValue);
            if (!_player.IsAlive())
            {
                _currentGameState = GameState.END; 
                Debug.Log("You Lost! You were killed by the Smashing Hammer!");
                return;
            }
            Debug.Log("You Successfully defended against the Smashing Hammer!");
        }
        else
        {
            Debug.Log("Smashing Hammer Attack failed! AI Smacked their Hand instead! -2HP | " + _ai.GetHealth() + "HP Left!");
            _ai.DealDamage(2);
            if (!_ai.IsAlive())
            {
                Debug.Log("AI was killed by their own Incompetence! You Won!");
                _currentGameState = GameState.END;
                return;
            }
        }
        _currentGameState = GameState.PRINT;
    }

    private void HandleAICharge()
    {
        _ai.SetCharging(true);
        Debug.Log("AI now Charging special Attack...");
        _currentGameState = GameState.PRINT;
    }
}
